@@ -12,7 +12,8 @@ from cytoolz.curried import (
     curry,
     partitionby,
 )
-from genomoncology.parse.doctypes import is_header
+
+from genomoncology.parse.doctypes import is_header, is_annotated_call
 from specd import create_app
 
 from genomoncology import kms
@@ -42,7 +43,7 @@ DEFAULT_SPECD_PATH = os.path.join(
     os.path.dirname(__file__), "../../gosdk/specs/"
 )
 
-VERSION = "0.9.15"
+VERSION = "0.9.16"
 COPYRIGHT = "Copyright: GenomOncology, LLC"
 
 
@@ -623,6 +624,128 @@ def match_therapies(state, diseases):
     ]
 
 
+@gocli.command()
+@options.pass_state
+@click.option(
+    "--diseases", "-d", help="Diseases", type=str, multiple=True, required=True
+)
+@click.option(
+    "--negative_alterations",
+    "-n",
+    help="Alterations that were not detected.",
+    type=str,
+    multiple=True,
+)
+@click.option(
+    "--indeterminate_alterations",
+    "-i",
+    help="List of alterations that are indeterminate "
+    "(i.e. considered not tested).",
+    type=str,
+    multiple=True,
+)
+@click.option(
+    "--evidence_category_filter",
+    "-ev",
+    help='Set to "exclude_preclinical" to exclude preclinical evidence',
+    type=str,
+)
+@click.option(
+    "--exclude_non1A_negative_matches",
+    "-ex",
+    help="Exclude wild-type non-1A negative therapy matches",
+    is_flag=True,
+)
+@click.option(
+    "--remove_nbm_trials_if_no_drug_target",
+    "-r",
+    help="Remove all non-biomarker trials from therapies that "
+    "have no gene targets or associations",
+    is_flag=True,
+)
+@click.option(
+    "--therapeutic_context_filter",
+    "-th",
+    help='Set to "exclude_adjuvant" to filter out therapies and trials '
+    "that have a therapeutic context of adjuvant or neoadjuvant",
+    type=str,
+)
+@click.option(
+    "--therapy_type_filter",
+    "-tt",
+    help='Set to "exclude_non_targeted_therapies" to filter out therapies '
+    "and trials that don’t have a highest level drug type of targeted "
+    "therapy or immunotherapy",
+    type=str,
+)
+@click.option(
+    "--gender",
+    "-g",
+    help="Filter results on gender. Possible values are Male or Female",
+    type=str,
+)
+@click.option(
+    "--date_of_birth",
+    "-dob",
+    help="Date of birth for match eligibility. Format: YYYY-MM-DD",
+    type=str,
+)
+@click.option(
+    "--countries",
+    "-c",
+    help="Filter results on countries",
+    type=str,
+    multiple=True,
+)
+@click.option(
+    "--snv_panel", "-snv", help="Genes on SNV panel.", type=str, multiple=True
+)
+@click.option(
+    "--cnv_panel", "-cnv", help="Genes on CNV panel.", type=str, multiple=True
+)
+@click.option(
+    "--cnv_panel_alterations",
+    "-cnv_a",
+    help="Alterations on CNV panel.",
+    type=str,
+    multiple=True,
+)
+@click.option(
+    "--ctx_panel",
+    "-ctx",
+    help="Alterations on CTX panel.",
+    type=str,
+    multiple=True,
+)
+@click.option(
+    "--karyotype_panel",
+    "-kary",
+    help="Alterations on Karyotype panel.",
+    type=str,
+    multiple=True,
+)
+@click.option(
+    "--biomarker_panel",
+    "-bio",
+    help="Alterations on Biomarker panel.",
+    type=str,
+    multiple=True,
+)
+@click.option(
+    "--prx_panel", "-prx", help="Genes on PRX panel.", type=str, multiple=True
+)
+@click.option(
+    "--assay", "-a", help="Name(s) of assays", type=str, multiple=True
+)
+def mega_match(state, **kwargs):
+    """Call mega-match using VIE response and diseases."""
+    return [
+        filter(is_annotated_call),
+        partition_all(99_999_999),
+        kms.create_sync_processor(state, kms.mega_match.mega_match, **kwargs),
+    ]
+
+
 @gocli.command()
 @click.option(
     "--diseases",
