@@ -59,6 +59,7 @@ class DocType(Enum):
     SECONDARY_FINDINGS_GENES = "SECONDARY_FINDINGS_GENES"
     MITO_TRNA = "MITO_TRNA"
     MTDB = "MTDB"
+    CLINGEN_DOSAGE = "CLINGEN_DOSAGE"
 
     def is_a(self, a_dict):
         try:
@@ -130,6 +131,10 @@ def is_header(doc):
     return DocType.HEADER.is_not(doc)
 
 
+def is_annotated_call(doc):
+    return DocType.ANNOTATED_CALL.is_a(doc)
+
+
 @curry
 def filter_out_ref_unknown(doc, keep_ref_unknown_calls=False):
     if not keep_ref_unknown_calls:
