@@ -290,7 +290,10 @@ class AggregatedCOSMICSources(LazyFileSource):
         self.log_file.write(
             "Step 1: Process the TSV file (parse and aggregate).\n"
         )
-        g_m_id_to_tsv_records = self.parse_tsv_records()
+        (
+            g_m_id_to_tsv_records,
+            cosmic_tsv_no_g_m_id_dict,
+        ) = self.parse_tsv_records()
 
         self.log_file.write(
             "Step 2: Loop through the VCF records and match "
@@ -357,26 +360,9 @@ class AggregatedCOSMICSources(LazyFileSource):
         )
 
         self.log_file.write(
-            "Seeing if there are any TSV/VCF records without a match.\n"
+            "Seeing if there are any VCF records without a match.\n"
         )
 
-        # print out any TSV records that are leftover
-        # that did not have any VCF matches. This is fine
-        # if this happens, but should be noted.
-        if g_m_id_to_tsv_records != {}:
-            self.log_file.write(
-                "There are TSV records that did not have a VCF record match.\n"
-            )
-            for g_m_id, aggregated_tsvs in g_m_id_to_tsv_records.items():
-                if len(aggregated_tsvs) > 0:
-                    tsvs_no_match_text = "\n".join(
-                        [str(t) for t in aggregated_tsvs]
-                    )
-                    self.log_file.write(tsvs_no_match_text)
-                    self.log_file.write("\n")
-        else:
-            self.log_file.write("All TSV records matched to a VCF record.\n")
-
         # See if there are any VCF records without a TSV match.
         # This indicates an error in the data and the script should stop.
         if len(vcfs_with_no_tsvs) > 0:
@@ -394,25 +380,96 @@ class AggregatedCOSMICSources(LazyFileSource):
 
         # At this point, we have a list of dictionaries, one per VCF line.
         # Each VCF line has been matched to a particular aggregated TSV
-        # grouping based on MUTATION_ID. Now, we need to do one final
-        # merge step where we merge these merged records together
-        # if they have the same CSRA. For the fields other than
-        # CHROM, POS, REF, and ALT, the field values will be lists
-        # with the data striped across.
+        # grouping based on GENOMIC_MUTATION_ID/LEGACY_ID/Gene.
+        # Now, we need to do one final merge step where we merge
+        # these merged records together if they have the same
+        # GENOMIC_MUTATION_ID.
         self.log_file.write(
-            "Step 3: Aggregating the merged records based on csra.\n"
+            "Step 3: Merge and yield records with the same "
+            "GENOMIC_MUTATION_ID.\n"
         )
-        csra_merged_records = self.aggregate_on_csra(merged_records)
+        g_m_id_merged_records = self.merge_records_with_g_m_id(merged_records)
 
         # yield these merged records
         self.log_file.write("Step 4: Yielding the merged records.\n")
-        for _, value in csra_merged_records.items():
+        for _, value in g_m_id_merged_records.items():
             if value.get("#CHROM"):
                 value["__type__"] = DocType.AGGREGATE.value
                 yield value
 
+        self.log_file.write(
+            "Step 4: Yield the TSV records that do not have a "
+            "GENOMIC_MUTATION_ID.\n"
+        )
+        for _, tsv_with_no_g_m_id in cosmic_tsv_no_g_m_id_dict.items():
+            tsv_with_no_g_m_id["AA"] = [
+                tsv_with_no_g_m_id.pop("Mutation AA", "")
+            ]
+            tsv_with_no_g_m_id["CDS"] = [
+                tsv_with_no_g_m_id.pop("Mutation CDS", "")
+            ]
+            tsv_with_no_g_m_id["MUTATION_ID"] = [
+                tsv_with_no_g_m_id.pop("MUTATION_ID", "")
+            ]
+            tsv_with_no_g_m_id["Gene name"] = [
+                tsv_with_no_g_m_id.pop("Gene name", "")
+            ]
+            tsv_with_no_g_m_id["LEGACY_ID"] = tsv_with_no_g_m_id.pop(
+                "LEGACY_MUTATION_ID", ""
+            )
+            tsv_with_no_g_m_id["__type__"] = DocType.AGGREGATE.value
+            # manually set hgvs_g to None so that it doesn't
+            # get calculated in the transform step
+            tsv_with_no_g_m_id["hgvs_g"] = None
+            yield tsv_with_no_g_m_id
+
         self.log_file.close()
 
+    def merge_records_with_g_m_id(self, tsv_vcf_merged_records):
+        merged_records = {}
+        for tsv_vcf_merged_record in tsv_vcf_merged_records:
+            g_m_id = tsv_vcf_merged_record.get("GENOMIC_MUTATION_ID")
+            existing_merged_record = merged_records.get(g_m_id, {})
+            merged_records[g_m_id] = self.do_merge_records(
+                existing_merged_record, tsv_vcf_merged_record
+            )
+        return merged_records
+
+    def do_merge_records(
+        self, existing_record, new_record
+    ):  # pragma: no mccabe
+        # The main three fields that will differ for each record that
+        # we are aggregating together are MUTATION_ID, CDS, and AA.
+        # Let's stripe these values.
+        striped_fields = ["MUTATION_ID", "CDS", "AA", "Gene name"]
+        for field in striped_fields:
+            existing_record[field] = existing_record.get(field, []) + [
+                new_record.get(field)
+            ]
+        # We may also have other fields whose values differ, but this is
+        # likely just bad data in cosmic, not actual differences. In this case
+        # we will overwrite the field values with the info from the record with
+        # the highest value of CNT.
+        fields_that_may_differ = [
+            "CNT",
+            "LEGACY_ID",
+            "TISSUES",
+            "TISSUES_FREQ",
+            "RESISTANCE_MUTATION",
+        ]
+        if "CNT" not in existing_record or (
+            "CNT" in existing_record
+            and new_record.get("CNT") > existing_record["CNT"]
+        ):
+            # we need to overwrite the values!
+            for field in fields_that_may_differ:
+                existing_record[field] = new_record.get(field)
+        # set the rest of the fields
+        for field_name, field_value in new_record.items():
+            if field_name not in striped_fields + fields_that_may_differ:
+                existing_record[field_name] = field_value
+        return existing_record
+
     def create_header(self):
         return {
             __TYPE__: DocType.HEADER.value,
@@ -421,23 +478,6 @@ class AggregatedCOSMICSources(LazyFileSource):
             "file_path": self.name,
         }
 
-    def aggregate_on_csra(self, merged_records):
-        # This method merges the aggregated VCF/TSV records together
-        # if they have the same csra.
-
-        aggregated_records = {}
-        for i in range(len(merged_records)):
-            merged_record = merged_records[i]
-            key = self.merge_key_value(merged_record)
-            aggregated_record = aggregated_records.setdefault(key, {})
-            aggregated_record = self.merge_aggregate_value(
-                aggregated_record, merged_record
-            )
-            aggregated_records[key] = aggregated_record
-            merged_record = None
-            merged_records[i] = None
-        return aggregated_records
-
     def merge_vcf_with_tsv(self, vcf_record, g_m_id_to_tsvs):
         # For each row in the VCF file, find a matching TSV
         # record. To "match", the GENE/Gene name values need to match,
@@ -481,7 +521,10 @@ class AggregatedCOSMICSources(LazyFileSource):
 
         self.log_file.write("Parsing the TSV file.\n")
 
-        cosmic_tsv_dict = self.aggregate_cosmic_tsv()
+        (
+            cosmic_tsv_dict,
+            cosmic_tsv_no_g_m_id_dict,
+        ) = self.aggregate_cosmic_tsv()
 
         self.log_file.write(
             "Processing the tissue frequencies for the TSVs.\n"
@@ -489,6 +532,9 @@ class AggregatedCOSMICSources(LazyFileSource):
 
         # add tissue frequency
         cosmic_tsv_dict = self.process_tissue_freqs(cosmic_tsv_dict)
+        cosmic_tsv_no_g_m_id_dict = self.process_tissue_freqs(
+            cosmic_tsv_no_g_m_id_dict
+        )
 
         # Group the aggregated TSV records by GENOMIC_MUTATION_ID.
         # This is basically just done to speed up performance in the
@@ -501,9 +547,11 @@ class AggregatedCOSMICSources(LazyFileSource):
             "for a quicker aggregation with the VCF files.\n"
         )
         g_m_id_to_tsv_records = self.group_tsv_by_g_m_id(cosmic_tsv_dict)
-        return g_m_id_to_tsv_records
+        return g_m_id_to_tsv_records, cosmic_tsv_no_g_m_id_dict
 
-    def cosmic_key_value(self, element):
+    def cosmic_key_value(self, element, is_tsv_no_g_m_id=False):
+        if is_tsv_no_g_m_id:
+            return element["LEGACY_MUTATION_ID"]
         return element["MUTATION_ID"], element["GENOMIC_MUTATION_ID"]
 
     def group_tsv_by_g_m_id(self, cosmic_tsv_dict):
@@ -532,16 +580,18 @@ class AggregatedCOSMICSources(LazyFileSource):
         )
 
         cosmic_tsv_dict = {}
+        cosmic_tsv_no_g_m_id_dict = {}
         for tsv_record in cosmic_tsv_source:
-            if tsv_record.get("GENOMIC_MUTATION_ID") in [
-                "",
-                None,
-                "GENOMIC_MUTATION_ID",
-            ]:
-                continue
-
-            key = self.cosmic_key_value(tsv_record)
-            aggregated_tsv_record = cosmic_tsv_dict.setdefault(
+            # Note: if there is no GENOMIC_MUTATION_ID, we will
+            # aggregate on LEGACY_MUTATION_ID instead.
+            is_tsv_no_g_m_id = tsv_record["GENOMIC_MUTATION_ID"] == ""
+            merged_dict = (
+                cosmic_tsv_no_g_m_id_dict
+                if is_tsv_no_g_m_id
+                else cosmic_tsv_dict
+            )
+            key = self.cosmic_key_value(tsv_record, is_tsv_no_g_m_id)
+            aggregated_tsv_record = merged_dict.setdefault(
                 key,
                 {
                     "CNT": 0,
@@ -550,14 +600,72 @@ class AggregatedCOSMICSources(LazyFileSource):
                     "RESISTANCE_MUTATION": {},
                 },
             )
-            aggregated_tsv_record = self.do_aggregation(
-                aggregated_tsv_record, tsv_record
+            aggregated_tsv_record = self.aggregate(
+                aggregated_tsv_record, tsv_record, is_tsv_no_g_m_id
             )
-            cosmic_tsv_dict[key] = aggregated_tsv_record
+            merged_dict[key] = aggregated_tsv_record
 
-        return cosmic_tsv_dict
+        return cosmic_tsv_dict, cosmic_tsv_no_g_m_id_dict
 
-    def do_aggregation(self, agg, x):  # pragma: no mccabe
+    def handle_tsv_no_g_m_id(self, agg, x):
+        # add MUTATION_ID to the aggregated dict
+        mutation_id = x.get("MUTATION_ID")
+        if "MUTATION_ID" not in agg:
+            agg["MUTATION_ID"] = mutation_id
+        else:
+            # throw exception if the mutation_id for this row
+            # does not match the mutation_id previously found
+            # for this mutation ID
+            if mutation_id != agg["MUTATION_ID"]:
+                raise Exception(
+                    f"TSV data error for record with no "
+                    f"GENOMIC_MUTATION_ID. "
+                    f"The record with LEGACY_MUTATION_ID "
+                    f"{x.get('LEGACY_MUTATION_ID')} "
+                    f"has more than one value for MUTATION_ID. "
+                    f"Values found are: "
+                    f"{agg['Mutation ID']} and {mutation_id}."
+                )
+
+        # add Mutation AA to the aggregated dict
+        mutation_aa = x.get("Mutation AA")
+        if "Mutation AA" not in agg:
+            agg["Mutation AA"] = mutation_aa
+        else:
+            # throw exception if the mutation_aa for this row
+            # does not match the mutation_aa previously found
+            # for this mutation ID
+            if mutation_aa != agg["Mutation AA"]:
+                raise Exception(
+                    f"TSV data error for record with no "
+                    f"GENOMIC_MUTATION_ID. "
+                    f"The record with LEGACY_MUTATION_ID "
+                    f"{x.get('LEGACY_MUTATION_ID')} "
+                    f"has more than one value for Mutation AA. "
+                    f"Values found are: "
+                    f"{agg['Mutation AA']} and {mutation_aa}."
+                )
+
+        # add Mutation CDS to the aggregated dict
+        mutation_cds = x.get("Mutation CDS")
+        if "Mutation CDS" not in agg:
+            agg["Mutation CDS"] = mutation_cds
+        else:
+            # throw exception if the mutation_cds for this row
+            # does not match the mutation_aa previously found
+            # for this mutation ID
+            if mutation_cds != agg["Mutation CDS"]:
+                raise Exception(
+                    f"TSV data error for record with no "
+                    f"GENOMIC_MUTATION_ID. "
+                    f"The record with LEGACY_MUTATION_ID "
+                    f"{x.get('LEGACY_MUTATION_ID')} "
+                    f"has more than one value for Mutation CDS. "
+                    f"Values found are: "
+                    f"{agg['Mutation CDS']} and {mutation_cds}."
+                )
+
+    def aggregate(self, agg, x, is_tsv_no_g_m_id=False):  # pragma: no mccabe
         # add gene name to the aggregated dict
         gene_name = x.get("Gene name")
         if "Gene name" not in agg:
@@ -589,6 +697,9 @@ class AggregatedCOSMICSources(LazyFileSource):
                     f"and {l_m_id}."
                 )
 
+        if is_tsv_no_g_m_id:
+            self.handle_tsv_no_g_m_id(agg, x)
+
         # update the counts for the tissue sites and resistance mutations
         agg["CNT"] += 1
         if x.get("Primary site") in agg["TISSUES"]:
@@ -639,35 +750,6 @@ class AggregatedCOSMICSources(LazyFileSource):
         else:
             v["Gene"] = "None"
 
-    def merge_key_value(self, element):
-        return "-".join(
-            [element["#CHROM"], element["POS"], element["REF"], element["ALT"]]
-        )
-
-    def merge_aggregate_value(self, agg, x):
-        # these fields are single_valued
-        for key in ["#CHROM", "POS", "REF", "ALT"]:
-            if key not in agg:
-                agg[key] = x[key]
-        # these fields will all be striped across all the merged records
-        for key in [
-            "GENE",
-            "CNT",
-            "MUTATION_ID",
-            "ID",  # this is the same as GENOMIC_MUTATION_ID in TSV
-            "TISSUES",
-            "TISSUES_FREQ",
-            "RESISTANCE_MUTATION",
-            "CDS",
-            "AA",
-            "LEGACY_ID",  # this is the same as LEGACY_MUTATION_ID in TSV
-        ]:
-            if key not in agg:
-                agg[key] = [x[key]]
-            else:
-                agg[key] = agg[key] + [x[key]]
-        return agg
-
 
 COSMIC_TSV_COLUMNS = [
     "Gene name",
