@@ -51,10 +51,22 @@ def construct_sdk(  # pragma: no mccabe
     max_time=MAX_TIME,
     max_tries=12,
     timeout=300,
+    custom_header_option=None,
 ):
 
     token = token or KMS_TOKEN
-    headers = dict(Authorization=f"Token {token}") if token else None
+
+    # add custom headers to the current headers var
+    # creates a dict of key:val by iterating through
+    # the tuple of custom_header_option and initializes
+    # the dict by separating each element in the tuple
+    # by ':'. ex. ('key1:val1','key2:val2') will be
+    # {'key1':'val1','key2:'val2'}
+
+    headers = dict(map(lambda x: x.split(':'), custom_header_option or []))
+    if token:
+        headers["Authorization"] = f"Token {token}"
+
     specd_path = specd_path or DEFAULT_SPECD_PATH
 
     config = config or {}
